/* -*- c++ -*-

This is Picprog, pic16c84 programmer software for the serial port device.
Copyright  1997 Jaakko Hyvtti

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

The author may be contacted at:

Email: Jaakko.Hyvatti@iki.fi
URL:   http://www.iki.fi/hyvatti/
Phone: +358 40 5011222

Please send any suggestions, bug reports, success stories etc. to the
Email address above.

*/

/*

  Hexfile follows the Microchip hex file format for pic16c84:

  - program memory at locations 0-0x3ff
  - id at locations 0x2000-0x2003
  - fuses at location 0x2007
  - data memory at locations 0x2100-0x213f

 */


#ifndef H_HEXFILE
#define H_HEXFILE

#include <fstream.h>

#include "picport.h"

class hexfile {
  const int size = 0x4000;
  short pgm [size];

  int program_location (picport& pic, int addr, bool data) const;

public:
  enum formats { unknown = 0, ihx8m, ihx16 };

private:
  void save_line (ofstream& f, int begin, int len, enum formats format) const;
  void save_region (ofstream& f, int addr, int end, enum formats format, bool skip_ones) const;
public:

  hexfile () {
    int i;
    for (i = 0; i < size; i++)
      pgm [i] = -1;
  }

  int load (const char *name);
  int save (const char *name, enum formats format, bool skip_ones) const;

  int program (const char *port, bool erase) const;
  int read (const char *port);

  void set (int addr, int value) {
    if (addr >= 0 && addr < size)
      pgm [addr] = value & 0x3fff;
  }
  int get (int addr) {
    if (addr < 0 || addr >= size)
      return -1;
    return pgm [addr];
  }
};

#endif // H_HEXFILE
