/* -*- c++ -*-

This is Picprog, pic16c84 programmer software for the serial port device.
Copyright  1997 Jaakko Hyvtti

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

The author may be contacted at:

Email: Jaakko.Hyvatti@iki.fi
URL:   http://www.iki.fi/hyvatti/
Phone: +358 40 5011222

Please send any suggestions, bug reports, success stories etc. to the
Email address above.

*/

#include <iostream.h>
#include <sysexits.h>
#include <unistd.h>
#include <getopt.h>

#include "hexfile.h"
#include "program.h"


program prog;

int opt_warranty = 0;
int opt_copying = 0;
int opt_quiet = 0;
int opt_usage = 0;

int opt_format = hexfile::ihx16;
char *opt_port = "/dev/ttyS0";
char *opt_input = NULL;
char *opt_output = NULL;
char *opt_cc = NULL;
int opt_skip = 0;
int opt_erase = 0;
int opt_burn = 0;

struct option long_opts[] = {
  {"quiet", no_argument, NULL, 'q'},
  {"warranty", no_argument, &opt_warranty, 1},
  {"copying", no_argument, &opt_copying, 1},
  {"help", no_argument, NULL, 'h'},
  {"pic-serial-port", required_argument, NULL, 'p'},
  {"input-hexfile", required_argument, NULL, 'i'},
  {"output-hexfile", required_argument, NULL, 'o'},
  {"ihx16", no_argument, &opt_format, hexfile::ihx16},
  {"ihx8m", no_argument, &opt_format, hexfile::ihx8m},
  {"cc-hexfile", required_argument, NULL, 'c'},
  {"skip-ones", no_argument, &opt_skip, 1},
  {"erase", no_argument, &opt_erase, 1},
  {"burn", no_argument, &opt_burn, 1},
  {NULL}
};

char short_opts [] = "p:i:o:c:qh?";



int
main (int argc, char **argv)
{
  int optc;

  prog.init (argv);

  while (0 <= (optc = getopt_long (argc, argv, short_opts, long_opts, NULL)))
    switch (optc) {
    case 0:
      break;
    case 'p':
      opt_port = optarg;
      break;
    case 'i':
      opt_input = optarg;
      break;
    case 'o':
      opt_output = optarg;
      break;
    case 'c':
      opt_cc = optarg;
      break;
    case 'q':
      opt_quiet = 1;
      break;
    default: // -? -h --help unknown flag
      opt_usage = 1;
    }

  if (!opt_quiet || opt_warranty || opt_copying || opt_usage)
    cerr << "Picprog version 1.0, Copyright  1997 Jaakko Hyvtti\n"
      "Picprog comes with ABSOLUTELY NO WARRANTY; for details\n"
      "type `" << prog.name << " --warranty'.  This is free software,\n"
      "and you are welcome to redistribute it under certain conditions;\n"
      "type `" << prog.name << " --copying' for details.\n\n";

  if (opt_copying)
    prog.copying ();

  if (opt_warranty)
    prog.warranty ();

  if (opt_usage) {
    cerr << "Full documentation is at "
      "<URL:http://www.iki.fi/hyvatti/pic/picprog.html>.\n"
      "The author may be contacted at:\n\n"

      "Email: Jaakko.Hyvatti@iki.fi\n"
      "URL:   http://www.iki.fi/hyvatti/\n"
      "Phone: +358 40 5011222\n\n";

    prog.usage (long_opts, short_opts);
  }

  if (opt_warranty || opt_copying || opt_usage)
    return EX_OK;

  if (!opt_input && !opt_output) {
    cerr << "Please specify either input or output hexfile.\n";
    prog.usage (long_opts, short_opts);
  }

  // if both input and output files are specified, first program the device
  // and then read it.

  if (opt_input) {
    hexfile mem;
    int retval;

    if (EX_OK != (retval = mem.load (opt_input)))
      return retval;
    if (opt_burn) {
      if (EX_OK != (retval = mem.program (opt_port, opt_erase)))
	return retval;
    } else
      cout << "No --burn option specified, device not programmed\n";

    if (opt_cc)
      if (EX_OK != (retval = mem.save (opt_cc, hexfile::formats (opt_format),
				       opt_skip)))
	return retval;
  }

  if (opt_output) {
    hexfile mem;
    int retval;

    if (EX_OK != (retval = mem.read (opt_port)))
      return retval;
    if (EX_OK != (retval = mem.save (opt_output,
				     hexfile::formats (opt_format),
				     opt_skip)))
      return retval;
  }

  return EX_OK;
}
