/*

This is a generic string class v1.0.
Copyright  1997 Jaakko Hyvtti

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software Foundation,
Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

The author may be contacted at:

Email: Jaakko.Hyvatti@iki.fi
URL:   http://www.iki.fi/hyvatti/
Phone: +358 40 5011222

Please send any suggestions, bug reports, success stories etc. to the
Email address above.

*/

#include <ctype.h>

#include "stringc.h"

string& string::operator= (const unsigned char *s1)
{
  if (s != s1) {
    if (s1) {
      size_t newlen = strlen ((const char *)s1);

      if (!s || newlen+1 > allocated) {
	if (s)
	  delete[] s;
	s = new unsigned char [allocated = newlen+1];
      }
      len = newlen;
      memcpy (s, s1, len);
      s [len] = '\0';
    } else {
      if (s) {
	delete[] s;
	s = 0;
      }
    }
  }
  return *this;
}

string& string::operator= (const string& s1)
{
  if (this != &s1) {
    if (s1.s) {
      if (!s || s1.len+1 > allocated) {
	if (s)
	  delete[] s;
	s = new unsigned char [allocated = s1.len+1];
      }
      memcpy (s, s1.s, s1.len);
      len = s1.len;
      s [len] = '\0';
    } else {
      if (s) {
	delete[] s;
	s = 0;
      }
    }
  }
  return *this;
}

size_t string::allocate (size_t newlen)
{
  if (!s || newlen+1 > allocated) {
    unsigned char *s1 = new unsigned char [allocated = newlen+1];

    if (s) {
      memcpy (s1, s, len);
      delete[] s;
    } else
      len = 0;
    s1 [len] = '\0';
    s = s1;
  }
  return allocated-1;
}

// this could be done more efficient, but the implementation below is neat..

string operator+ (const string& s1, const string& s2)
{
  string retval (s1.length() + s2.length());

  retval = s1;
  retval += s2;
  return retval;
}

string& string::operator+= (const string& s1)
{
  // We do not protect against this==s1, 'cause it should work..

  if (s1.s) {
    if (!s)
      return operator= (s1);
    if (len + s1.len + 1 > allocated) {
      unsigned char *s3 = new unsigned char [allocated = len + s1.len + 1];
      memcpy (s3, s, len);
      delete[] s;
      s = s3;
    }
    memcpy (s + len, s1.s, s1.len);
    len += s1.len;
    s [len] = '\0';
  }
  return *this;
}

string& string::operator= (const int c)
{
  if (!s || 1+1 > allocated) {
    if (s)
      delete[] s;
    s = new unsigned char [allocated = 1+1];
  }
  s [0] = c;
  s [len = 1] = '\0';
  return *this;
}

string& string::operator+= (const int c)
{
  if (!s)
    return operator= (c);
  if (len + 1 + 1 > allocated) {
    unsigned char *s3 = new unsigned char [allocated = len + 1 + 1];
    memcpy (s3, s, len);
    delete[] s;
    s = s3;
  }
  s [len++] = c;
  s [len] = '\0';
  return *this;
}


// outputs a string including any null characters

ostream& operator<< (ostream& o, const string& z)
{
  int len = z.length ();
  int i = 0;

  while (len--)
    o.put (z[i++]);
  return o;
}

istream& operator>> (istream& i, string& z)
{
  char buf [256];
  size_t got = 0;
  int ws = 1;
  int first = 1;
  char c;

  while (i.get (c)) {
    if (isspace (c))
      if (ws)
	continue;
      else {
	i.putback (c);
	break;
      }
    ws = 0;
    buf [got++] = c;
    if (got >= sizeof (buf)) {
      if (first) {
	z = string (buf, got);
	first = 0;
      } else
	z += string (buf, got);
      got = 0;
    }
  }
  if (first)
    z = string (buf, got);
  else
    if (got)
      z += string (buf, got);
  return i;
}

void string::debug () const
{
  cout << "DEBUG:string:" << hex << (unsigned int)(this) << ", s:" <<
    (unsigned int)(s) << ", l:" << dec << len << ", a:" << allocated <<
    ", ..:" << (s ? int (s[0]) : 0) << endl;

}

bool operator< (const string& s1, const string& s2)
{
  if (s1.length () < s2.length ()) {
    if (s1.length ())
      return memcmp (s1.asciz (), s2.asciz (), s1.length ()) <= 0;
    return 1;
  }
  if (s2.length ())
    return memcmp (s1.asciz (), s2.asciz (), s1.length ()) < 0;
  return 0;
}


